<?php
// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Main class for cusom app actions.
 */
class FlowMattic_Custom_User_App {
	/**
	 * Constructor.
	 *
	 * @access public
	 * @since 3.0
	 * @return void
	 */
	public function __construct() {
	}

	/**
	 * Run the action step.
	 *
	 * @access public
	 * @since 3.0
	 * @param string $workflow_id  Workflow ID.
	 * @param object $step         Workflow current step.
	 * @param array  $capture_data Data captured by the WordPress action.
	 * @return array
	 */
	public function run_action_step( $workflow_id, $step, $capture_data ) {
		$step   = (array) $step;
		$action = $step['action'];
		$fields = isset( $step['fields'] ) ? $step['fields'] : ( isset( $step['actionAppArgs'] ) ? $step['actionAppArgs'] : array() );

		// Check response type.
		$simple_response = ( ! isset( $fields['simple_response'] ) ) ? 'Yes' : $fields['simple_response'];

		// Get the App ID.
		$app_id = $step['application'];

		// Get the app data.
		$flowmattic_apps = wp_flowmattic()->apps;
		$all_apps        = $flowmattic_apps->get_other_action_applications();
		$app_data        = $all_apps[ $app_id ];

		// Get the app settings.
		$app_settings = isset( $app_data['app_settings'] ) ? maybe_unserialize( $app_data['app_settings'] ) : array();

		// Check if needs custom ID.
		$needs_connect = isset( $app_data['needs_connect'] ) ? true : false;

		// Get connect ID used for the app.
		if ( $needs_connect ) {
			$connect_id = isset( $fields['connect_id'] ) ? $fields['connect_id'] : '';
		} else {
			$connect_id = isset( $app_settings['connect_id'] ) ? $app_settings['connect_id'] : '';
		}

		// Get the available actions.
		$app_actions = isset( $app_data['actions'] ) ? maybe_unserialize( $app_data['actions'] ) : array();

		// Check if the action event is availabe, get the data if available.
		$app_action = ( is_array( $app_actions ) && isset( $app_actions[ $action ] ) ) ? $app_actions[ $action ]['action_data'] : array();

		// Get the request type.
		$request_type = $app_action['http_method'];

		$body         = array();
		$headers      = array(
			'User-Agent: FlowMattic/' . FLOWMATTIC_VERSION,
		);
		$request_args = array(
			'body'        => $body,
			'headers'     => array(
				'User-Agent'   => 'FlowMattic/' . FLOWMATTIC_VERSION,
				'Accept'       => 'application/json',
				'Content-Type' => 'application/json',
			),
			'timeout'     => 60,
			'sslverify'   => false,
			'data_format' => 'body',
			'method'      => strtoupper( $request_type ),
		);

		// Trim the space for endpoint url.
		$api_endpoint = trim( $app_action['endpoint_url'] );

		if ( '' !== $connect_id ) {
			if ( 'none' !== $connect_id ) {
				// Get the connect data.
				$connect_args = array(
					'connect_id' => $connect_id,
				);

				// Get the connect data from db.
				$connect = wp_flowmattic()->connects_db->get( $connect_args );

				// Check if external connect.
				$external_connect = ( isset( $connect->connect_settings['is_external'] ) ) ? flowmattic_get_connects( $connect->connect_settings['external_slug'] ) : false;

				// Set the authorization according to the auth type.
				switch ( $connect->connect_settings['fm_auth_type'] ) {
					case 'oauth':
						$connect_data = $connect->connect_data;

						// Add authentication.
						$request_args['headers']['Authorization'] = 'Bearer ' . $connect_data['access_token'];

						// Headers used in cURL request.
						$headers[] = 'Authorization: Bearer ' . $connect_data['access_token'];

						break;

					case 'bearer':
						$token = $connect->connect_settings['auth_bearer_token'];

						// Add authentication to header.
						$request_args['headers']['Authorization'] = 'Bearer ' . $token;

						// Headers used in cURL request.
						$headers[] = 'Authorization: Bearer ' . $token;

						break;

					case 'basic':
						$api_key    = $connect->connect_settings['auth_api_key'];
						$api_secret = $connect->connect_settings['auth_api_secret'];

						$request_args['headers']['Authorization'] = 'Basic ' . base64_encode( $api_key . ':' . $api_secret ); // @codingStandardsIgnoreLine

						// Headers used in cURL request.
						$headers[] = 'Authorization: Basic ' . base64_encode( $api_key . ':' . $api_secret ); // @codingStandardsIgnoreLine

						break;

					case 'api':
						$add_to    = ! empty( $external_connect ) ? $external_connect['auth_api_addto'] : $connect->connect_settings['auth_api_addto'];
						$api_key   = ! empty( $external_connect ) ? $external_connect['auth_api_key'] : $connect->connect_settings['auth_api_key'];
						$api_value = $connect->connect_settings['auth_api_value'];

						if ( 'query' === $add_to ) {
							$api_endpoint = add_query_arg( $api_key, $api_value, $api_endpoint );
						} else {
							$request_args['headers'][ $api_key ] = trim( $api_value );

							// Headers used in cURL request.
							$headers[] = $api_key . ': ' . trim( $api_value );
						}

						break;
				}
			}
		} else {
			// If there's no response, return default message.
			return wp_json_encode(
				array(
					'status'  => esc_attr__( 'Error', 'flowmattic' ),
					'message' => esc_html__( 'Connect ID not provided', 'flowmattic' ),
				)
			);
		}

		// If action requires custom headers, set them all.
		if ( isset( $app_action['add_headers'] ) && 'no' !== $app_action['add_headers'] ) {
			$api_headers = $app_action['dynamic-headers-key'];

			foreach ( $api_headers as $header_id => $header_key ) {
				if ( isset( $fields[ $header_key ] ) ) {
					$header_value = $fields[ $header_key ];

					// Headers used in regular requests.
					$request_args['headers'][ $header_key ] = trim( stripslashes( $header_value ) );

					// Headers used in cURL request.
					$headers[] = $header_key . ': ' . trim( stripslashes( $header_value ) );
				}
			}
		}

		// Get the content type header from app.
		$content_type = $app_action['content_type'];

		// Get the content type value.
		$content_type = $this->get_content_types()[ $content_type ]['type'];

		// Set the content type header.
		$request_args['headers']['Content-Type'] = $content_type;
		$headers[]                               = 'Content-Type: ' . $content_type;

		// Check if action has set raw data.
		$raw_data = stripslashes( $app_action['raw_data'] );

		// If action has body params, set them all.
		if ( isset( $app_action['add_params'] ) && 'no' !== $app_action['add_params'] ) {
			$api_params    = $app_action['dynamic-params-key'];
			$param_options = $app_action['dynamic-params-options'];
			$body_params   = array();

			foreach ( $api_params as $param_id => $param_key ) {
				if ( isset( $fields[ $param_key ] ) ) {
					$param_option = json_decode( stripslashes( base64_decode( $param_options[ $param_id ] ) ) );
					$param_value  = trim( $fields[ $param_key ] );
					$replace_tag  = '{{' . $param_key . '}}';

					// Decode the URL endpoint to replace the dynamic tags.
					$api_endpoint = urldecode( $api_endpoint );

					if ( false !== strpos( $api_endpoint, $replace_tag ) ) {
						$api_endpoint = str_replace( $replace_tag, $param_value, $api_endpoint );
						$api_endpoint = urlencode( $api_endpoint );
						if ( 'no' !== $param_option->path_variable ) {
							continue;
						}
					}

					// If raw data is set, replace the dynamic values.
					if ( '' !== $raw_data ) {
						$raw_data = str_replace( $replace_tag, $param_value, $raw_data );
					}

					// Build params array.
					$body_params[ $param_key ] = stripslashes( $param_value );
				}
			}

			// If raw data is set, use it for request body.
			if ( '' !== $raw_data ) {
				$request_args['body'] = $raw_data;
			} elseif ( ! empty( $body_params ) ) {
				if ( 'get' === $request_type ) {
					$api_endpoint         = ( false === strpos( $api_endpoint, '?' ) ) ? $api_endpoint . '?' . http_build_query( $body_params ) : $api_endpoint . '&' . http_build_query( $body_params );
					$request_args['body'] = '';
				} else {
					if ( 'encoded_form_data' === $app_action['content_type'] ) {
						$request_args['body'] = http_build_query( $body_params );
					} elseif ( 'form_data' === $app_action['content_type'] ) {
						$request_args['body'] = $body_params;
					} elseif ( 'xml' === $app_action['content_type'] ) {
						$xml_data = new SimpleXMLElement( '<?xml version="1.0"?><data></data>' );

						foreach ( $body_params as $key => $value ) {
							$xml_data->addChild( $key, $value );
						}

						$xml_output = $xml_data->asXML();

						$request_args['body'] = $xml_output;
					} else {
						$request_args['body'] = wp_json_encode( $body_params );
					}
				}
			}
		}

		// Make the live request.
		$request_response = $this->request( $request_args, $api_endpoint, $headers, $simple_response );

		return $request_response;
	}

	/**
	 * Set content types.
	 *
	 * @access public
	 * @since 3.0
	 * @return array
	 */
	public function get_content_types() {
		return array(
			'json'              => array(
				'title' => esc_attr__( 'JSON - With Parameters', 'flowmattic' ),
				'type'  => 'application/json',
			),
			'custom_json'       => array(
				'title' => esc_attr__( 'Custom JSON - Use Your Code', 'flowmattic' ),
				'type'  => 'application/json',
			),
			'form_data'         => array(
				'title' => esc_attr__( 'Form Data', 'flowmattic' ),
				'type'  => 'multipart/form-data',
			),
			'encoded_form_data' => array(
				'title' => esc_attr__( 'URL Encoded Form Data', 'flowmattic' ),
				'type'  => 'application/x-www-form-urlencoded',
			),
			'text'              => array(
				'title' => esc_attr__( 'Text', 'flowmattic' ),
				'type'  => 'text/plain',
			),
			'html'              => array(
				'title' => esc_attr__( 'HTML', 'flowmattic' ),
				'type'  => 'text/html',
			),
			'xml'               => array(
				'title' => esc_attr__( 'XML', 'flowmattic' ),
				'type'  => 'application/xml',
			),
		);
	}

	/**
	 * Process the action request.
	 *
	 * @access public
	 * @since 3.0
	 * @param string $request_args Request params.
	 * @param string $endpoint     Endpoint URL.
	 * @return string
	 */
	public function request( $request_args, $endpoint, $headers, $simple_response ) {
		// Initialize cURL.
		$curl = curl_init( $endpoint );

		// Set cURL options.
		curl_setopt_array(
			$curl,
			array(
				CURLOPT_URL            => $endpoint,
				CURLOPT_CUSTOMREQUEST  => $request_args['method'],
				CURLOPT_POSTFIELDS     => $request_args['body'],
				CURLOPT_RETURNTRANSFER => true, // Return the response as a string.
				CURLOPT_FOLLOWLOCATION => true, // Follow redirects.
				CURLOPT_MAXREDIRS      => 5, // Maximum number of redirects.
				CURLOPT_TIMEOUT        => 15, // Timeout in seconds.
				CURLOPT_SSL_VERIFYHOST => 0,
				CURLOPT_SSL_VERIFYPEER => 0,
				CURLOPT_HTTPHEADER     => $headers,
			)
		);

		// Execute the request.
		$request_body   = curl_exec( $curl );
		$response_code  = curl_getinfo( $curl, CURLINFO_HTTP_CODE );
		$request_decode = json_decode( $request_body, true );

		$response_array = array();

		$response_status = ( $response_code >= 200 && $response_code <= 300 ) ? 'success' : 'error';
		if ( 'error' === $response_status ) {
			$response_array['status']   = 'error';
			$response_array['response'] = is_string( $request_body ) ? $request_body : wp_json_encode( $request_body );
		} else {
			$response_array['status'] = 'success';
		}

		if ( is_array( $request_decode ) ) {
			if ( isset( $request_decode[0] ) && is_array( $request_decode[0] ) && 'No' === $simple_response ) {
				$response_array['response'] = $request_body;
			} else {
				foreach ( $request_decode as $key => $value ) {
					if ( is_int( $key ) && 'No' === $simple_response ) {
						$response_array['response'] = wp_json_encode( $request_decode );
						break;
					}

					if ( is_array( $value ) && 'No' !== $simple_response ) {
						$response_array = flowmattic_recursive_array( $response_array, $key, $value );
					} else {
						$response_array[ $key ] = ( is_array( $value ) ) ? wp_json_encode( $value ) : $value;
					}
				}
			}
		}

		// Add response code to the request.
		$response_array['response_code'] = $response_code;

		if ( '' === $request_body ) {
			// If there's no response, return default message.
			return wp_json_encode(
				array(
					'status'   => esc_attr__( 'Success', 'flowmattic' ),
					'response' => esc_attr__( 'Request returned no response data', 'flowmattic' ),
				)
			);
		} else {
			return wp_json_encode( $response_array );
		}
	}

	/**
	 * Test action event ajax.
	 *
	 * @access public
	 * @since 3.0
	 * @param array $event_data Test event data.
	 * @return array
	 */
	public function test_event_action( $event_data ) {
		$event       = $event_data['event'];
		$fields      = isset( $event_data['fields'] ) ? $event_data['fields'] : ( isset( $settings['actionAppArgs'] ) ? $settings['actionAppArgs'] : array() );
		$workflow_id = $event_data['workflow_id'];

		// Replace action for testing.
		$event_data['action'] = $event;

		$request = $this->run_action_step( $workflow_id, $event_data, $fields );

		return $request;
	}
}

new FlowMattic_Custom_User_App();
